@@ -31,7 +31,7 @@ from qtpy.QtCore import Qt, QTimer
 from qtpy.QtWidgets import QFileDialog, QMessageBox, QPushButton, QApplication
 from opensesame_ide import FolderBrowserDockWidget, MenuBar
 from libqtopensesame.misc.translate import translation_context
-from pyqode.core import widgets
+from pyqode.core import widgets, panels
 from pyqode.core.api.utils import TextHelper
 from pyqode_extras.widgets import FallbackCodeEdit
 _ = translation_context(u'OpenSesameIDE', category=u'extension')
@@ -80,11 +80,14 @@ class OpenSesameIDE(BaseExtension):
         self._register_mimetypes()
         self._patch_behavior()
         self._scetw = widgets.SplittableCodeEditTabWidget(self.main_window)
-        self._scetw.main_tab_widget.cornerWidget().hide()
         self._scetw.fallback_editor = FallbackCodeEdit
         self._scetw.tab_name = u'OpenSesameIDE'
-        self._scetw.main_tab_widget.tab_closed.connect(self._on_editor_close)
         self._scetw.tab_bar_visible = cfg.opensesame_ide_show_tab_bar
+        self._patch_splitter(self._scetw)
+        # self._scetw.main_tab_widget.tab_closed.connect(self._on_editor_close)
+        # self._scetw.editor_created.connect(self._on_editor_created)
+        self._scetw.default_extension = \
+            lambda: cfg.opensesame_ide_default_extension
         self._add_ide_tab()
         self._dock_widgets = {}
         self._set_ignore_patterns()
@@ -115,7 +118,17 @@ class OpenSesameIDE(BaseExtension):
         self.extension_manager.fire('ide_jump_to_line', lineno=line_number)
         
     def event_ide_search_text(self, needle):
-        
+        """
+        desc:
+            Conducts a case-insensitive search for a needle in the current
+            editor. The search starts from the cursor position, or from the
+            start if no match was found after the cursor position.
+            
+        arguments:
+            needle:
+                desc:    The search needle.
+                type:    str
+        """
         editor = self._current_editor()
         if editor is None:
             return u''
@@ -124,14 +137,19 @@ class OpenSesameIDE(BaseExtension):
         except KeyError:
             return  # Panel is not installed
         cursor = editor.textCursor()
-        needle_pos = cursor.block().text().find(needle)
+        needle = needle.lower()
+        code = editor.toPlainText().lower()
+        # First search from current cursor position
+        needle_pos = code.find(needle, cursor.position())
+        # If the needle is not found, search from the start of the code
         if needle_pos < 0:
-            needle_pos = editor.toPlainText().find(needle)
+            needle_pos = code.find(needle)
+        # If the needle still not found, then it doesn't exist.
         if needle_pos < 0:
             oslogger.warning('failed to find needle in text')
             return
         cursor.clearSelection()
-        cursor.movePosition(cursor.StartOfBlock, cursor.MoveAnchor)
+        cursor.movePosition(cursor.Start, cursor.MoveAnchor)
         cursor.movePosition(
             cursor.NextCharacter,
             cursor.MoveAnchor,
@@ -149,10 +167,22 @@ class OpenSesameIDE(BaseExtension):
 
         cfg.opensesame_ide_show_tab_bar = show_tab_bar
         self._scetw.tab_bar_visible = show_tab_bar
+        
+    def event_register_editor(self, editor, mime_type=u'text/plain'):
+        """The IDE mode has a special panel that allows the extension to be
+        changed for empty unsaved buffers.
+        """
+        try:
+            editor.panels.get('ChangeExtensionPanel')
+        except KeyError:
+            editor.panels.append(
+                panels.ChangeExtensionPanel(),
+                panels.ChangeExtensionPanel.Position.TOP
+            )
 
-    def event_ide_new_file(self, source=None):
+    def event_ide_new_file(self, source=None, ext=None):
 
-        self.new_file()
+        self.new_file(ext=ext)
         if source is None:
             return
         self._current_editor().setPlainText(source)
@@ -233,7 +263,7 @@ class OpenSesameIDE(BaseExtension):
     def open_document(self, path):
 
         # First normalize the path and check if there's a custom handler for it
-        path = os.path.abspath(os.path.normcase(path))
+        path = os.path.abspath(os.path.normpath(path))
         ext = os.path.splitext(path)[1].lstrip('.').lower()
         handler = self.extension_manager.provide(
             'open_file_extension_{}'.format(ext)
@@ -258,7 +288,7 @@ class OpenSesameIDE(BaseExtension):
         for editor in self._scetw.widgets():
             if editor.file.path is None:
                 continue
-            if path == os.path.normpath(os.path.normcase(editor.file.path)):
+            if path == os.path.abspath(os.path.normpath(editor.file.path)):
                 editor.parent().parent().setCurrentWidget(editor)
                 editor.setFocus()
                 return
@@ -353,15 +383,11 @@ class OpenSesameIDE(BaseExtension):
         self._add_recent_folder(path)
         self._remember_open_folders()
 
-    def new_file(self):
+    def new_file(self, ext=None):
 
-        editor = self._current_splitter().create_new_document(
-            extension=cfg.opensesame_ide_default_extension
-        )
-        self.extension_manager.fire(
-            u'register_editor',
-            editor=editor
-        )
+        if not ext:
+            ext = cfg.opensesame_ide_default_extension
+        editor = self._current_splitter().create_new_document(extension=ext)
 
     def save_file(self):
 
@@ -389,9 +415,11 @@ class OpenSesameIDE(BaseExtension):
 
     def save_file_as(self):
 
+        current_splitter = self._current_splitter()
         editor = self._current_editor()
+        current_splitter.main_tab_widget.unpin(editor)
         from_path = editor.file.path if editor else ''
-        self._current_splitter().save_current_as()
+        current_splitter.save_current_as()
         editor = self._current_editor()
         if editor is None:
             return
@@ -719,7 +747,7 @@ class OpenSesameIDE(BaseExtension):
 
         haystack = []
         norm_paths = {
-            os.path.normpath(os.path.normcase(path))
+            os.path.abspath(os.path.normpath(path))
             for path in cfg.opensesame_ide_recent_folders.split(u';')
             if os.path.isdir(path)
         }
@@ -750,9 +778,7 @@ class OpenSesameIDE(BaseExtension):
         # If there are no child splitters, we use the regular split() method
         if not splitter.child_splitters:
             subsplitter = splitter.split(editor, direction)
-            subsplitter.main_tab_widget.tab_closed.connect(
-                self._on_editor_close
-            )
+            self._patch_splitter(subsplitter)
             self.extension_manager.fire(
                 u'register_editor',
                 editor=self._current_editor()
@@ -765,16 +791,14 @@ class OpenSesameIDE(BaseExtension):
         # pyqode.
         self.main_window.setUpdatesEnabled(False)
         subsplitter = splitter.split(editor, splitter.orientation(), index=1)
-        subsplitter.main_tab_widget.tab_closed.connect(self._on_editor_close)
+        self._patch_splitter(subsplitter)
         editor = self._current_editor()
         self.extension_manager.fire(
             u'register_editor',
             editor=editor
         )
         subsubsplitter = subsplitter.split(editor, direction, index=1)
-        subsubsplitter.main_tab_widget.tab_closed.connect(
-            self._on_editor_close
-        )
+        self._patch_splitter(subsubsplitter)
         editor = self._current_editor()
         self.extension_manager.fire(
             u'register_editor',
@@ -864,7 +888,7 @@ class OpenSesameIDE(BaseExtension):
         if not self.main_window.ui.action_onetabmode.isChecked():
             self.main_window.ui.action_onetabmode.trigger()
         self.tabwidget.add = self._patch_tabwidget_add(self.tabwidget.add)
-        self.tabwidget.tabCloseRequested.connect(self._on_tabwidget_close)
+        self.tabwidget.tab_removed.connect(self._on_tabwidget_close)
         self.tabwidget.shortcut_switch_left.setKey(u'')
         self.tabwidget.shortcut_switch_right.setKey(u'')
         # Create a custom menubar
@@ -916,6 +940,19 @@ class OpenSesameIDE(BaseExtension):
             return fnc(widget, *args, **kwargs)
 
         return inner
+    
+    def _patch_splitter(self, splitter):
+        
+        tw = splitter.main_tab_widget
+        tw.tab_closed.connect(self._on_editor_close)
+        tw.split_requested.disconnect(splitter.split)
+        tw.split_requested.connect(self._on_split_requested)
+        splitter.editor_created.connect(self._on_editor_created)
+        
+    def _on_split_requested(self, widget, orientation, index=None):
+        
+        widget.setFocus()
+        self._split(orientation)
 
     def _on_tabwidget_close(self, index):
 
@@ -1059,6 +1096,22 @@ class OpenSesameIDE(BaseExtension):
     def provide_ide_current_path(self):
 
         return self._current_path()
+        
+    def _on_extension_changed(self, ext):
+        
+        cfg.opensesame_ide_default_extension = ext
+        
+    def _on_editor_created(self, editor):
+        self.extension_manager.fire(
+            u'register_editor',
+            editor=editor
+        )
+        try:
+            p = editor.panels.get('ChangeExtensionPanel')
+        except KeyError:
+            pass
+        else:
+            p.extension_changed.connect(self._on_extension_changed)
 
     @with_editor
     def _current_tabwidget(self, editor):
