@@ -84,7 +84,7 @@ class FolderBrowser(FileSystemTreeView):
         self.clear_ignore_patterns()
         self.add_ignore_patterns(ide.ignore_patterns)
         self.set_root_path(os.path.normpath(path))
-        self.set_context_menu(FileSystemContextMenu())
+        self.set_context_menu(FileSystemContextMenu(self.main_window))
         self._watcher = QFileSystemWatcher()
         self._watcher.addPath(path)
         self._watcher.fileChanged.connect(self._on_file_changed)
@@ -165,6 +165,11 @@ class FolderBrowser(FileSystemTreeView):
             self._path,
             self._file_indexer.pid
         ))
+        self._ide.extension_manager.fire(
+            'register_subprocess',
+            pid=self._file_indexer.pid,
+            description='file_indexer:{}'.format(self._path)
+        )
         QTimer.singleShot(1000, self._check_file_indexer)
 
     def _check_file_indexer(self):
